/* rbexp7 - modified from rbexp5 to run mixed blocks of trial types */
/* scott cost 5/3/94++++++ */

#include <stdio.h>
#include <stdlib.h>
#include <conio.h>
#include <graphics.h>
#include <string.h>
#include <ctype.h>

/* prototypes */
void inst(int);
void collect_data(char[], int);
void show_practice(FILE*, int);
void presentation(char[], int);
void instructions();
void do_block(FILE*, FILE*, int);
void start_graphics();
extern void start_clock(...);
extern void stop_clock(...);

extern int ms_counter; /* millisecond timer from MSCLOCK */

/* program constants */
#define X 300 /* screen center */
#define Y 230
#define Refresh 17
#define Warm_Up 2 /* two trials to start each block */
#define MASK_TIME Refresh /* initial mask duration, ms */
#define LETT2_TIME 160 /* initial letter presentation time, ms */
#define LETT3_TIME 160 /* initial letter presentation time, ms */
#define Field_Color 2 /* green */
#define Target_Color 4 /* red */
#define Fix_Color 7 /* light grey */
#define Blank_Color 0 /* black */
#define MSG_COLOR 7 /* light grey */
#define Num_Letters 7
#define Num_Blocks 4
#define Num_Trials 40   /* # trials in normal block */
#define Pract_Trials 20 /* # trials in practice block */
#define Num_Targ 3
#define Adjust_Rate 4 /* number of control trials per time adjustment */
#define MAX_LINE 100  /* max line length from input file */

/* Serial letter presentation program.  Built for Elwyn Leek by Scott Cost */
/* Must be compiled with MSCLOCK.CPP, RSC custom millisecond timer. */
int main (
	int argc,
	char* argv[] )
{
	int ms_lett; /* variables for letter and mask duration, in ms */
	int ms_mask;
	FILE *infile, *outfile; /* convention:  command argument 1 is in, 2 is out */
	int block;

	start_clock();
	_setcursortype(_NOCURSOR);

	if ((infile = fopen(argv[1],"r"))==NULL) {
		printf ("%s:  (In)File Open Error.\n", argv[1]);
		getch();
		exit(1);
	}
	if ((outfile = fopen(argv[2], "w"))==NULL)
	{
		printf ("%s:  (Out)File Open Error.\n", argv[2]);
		getch();
		exit(1);
	}
	inst(1);
	for (block=-2; block<Num_Blocks; block++) {
		clrscr();
		do_block(infile, outfile, block);
		clrscr();
		if (block>=0) {
			printf ("You have just completed block %d of 6.\n", block+3);
			printf ("Please wait . . .\n");
		}
		ms_counter=0;
		while (ms_counter<5000); /* pause for 5 seconds */
		/* process data */
	}
	clrscr();
	printf("This experiment is complete.  Thank you for your participation.\n");
	getch();
	fcloseall();
	stop_clock(); /* add this to an error handling routine */
	return(0); /* no error */
 }

void do_block (
	FILE *infile,
	FILE *outfile,
	int block
)
{
	int color[Num_Trials][Num_Letters];   /* color array */
	char letter[Num_Trials][Num_Letters]; /* character array */
	char response[Num_Trials][Num_Targ];  /* subject responses */
	int trial, itrial, position, i, swap, dest, num_trials;
	char line[MAX_LINE];
	char string[100]; /* presentation window */
	static int lett2_time = LETT2_TIME, lett3_time = LETT3_TIME;
	int lett_time, num_targ; /* current trial variables */
	int shuffle[Num_Trials];
	char a1, a2, a3, l1, l2, l3, type;
	int one, two, thr, score_l2, score_l3, score_s2, score_s3;
	float performance;
	int control=0, control_score=0, ldl;
	int ctrl_lll = 0, ctrl_ldl = 0; /* hold control counts for mixed cases */
	int s_scr_ldl = 0, s_scr_lll = 0;
	static int id=0;
	int ident[Num_Trials];

	start_graphics();


	if (block<0) {
		num_trials=Pract_Trials;
		show_practice(infile, block);
	}
		else num_trials=Num_Trials;

	closegraph(); /* insulate text screen */

	/*******************************************/

	if (block==-2) inst(2);
	else if (block==-1) inst(4);
	else if (block==0) inst(6);
	else if (block==1) inst(7);
	else if (block==2) inst(8);
	else if (block==3) inst(9);

	/* randomization - shuffle is the index for order of presentation */
	randomize(); /* time function dependent */
	for (i=0; i<num_trials; i++) shuffle[i]=i;
	for (i=0; i<num_trials; i++) { /* swap-shuffle */
		dest=random(num_trials);
		swap=shuffle[i]; shuffle[i]=shuffle[dest]; shuffle[dest]=swap;
	}

	for (trial=0; trial<num_trials; trial++) {
		ident[trial]=id++;
		fgets(line,MAX_LINE,infile);
		position = 0;
		letter[trial][position] = (strtok(line, " \t\n"))[0];

		for (position=1; position<Num_Letters; position++) {
			letter[trial][position] = (strtok(NULL, " \t\n"))[0];
		}
	}
	start_graphics();
	/*******************************************/

	for (trial=0; trial<num_trials; trial++) {

		itrial=shuffle[trial];

		/***** block mixture setting follows *****/
		/***** note - previously above...    *****/

		/* context set */
		if ((isdigit(letter[itrial][0])) | (isdigit(letter[itrial][1])) | (isdigit(letter[itrial][2])) |
				(isdigit(letter[itrial][3])) | (isdigit(letter[itrial][4])) | (isdigit(letter[itrial][5])) |
				(isdigit(letter[itrial][6]))) {
			lett_time=lett2_time; num_targ=3; ldl=1;
			}
		else {lett_time=lett3_time; num_targ=3; ldl=0;
		}

	/***** end block mixture setting *****/



		presentation(letter[itrial], lett_time);
		collect_data(response[itrial], block);

		/* process trial data */
		if (islower(letter[itrial][2]) | islower(letter[itrial][4])) type='F';
		else if (letter[itrial][2] == letter[itrial][4]) type='R';
		else type='C';
		fprintf (outfile, "%2d %d %c %d", block, trial, type, num_targ);
		fprintf (outfile, " <%c %c %c %c %c %c %c>", letter[itrial][0],
			letter[itrial][1], letter[itrial][2], letter[itrial][3],
			letter[itrial][4], letter[itrial][5], letter[itrial][6] );
		fprintf (outfile, " <%c %c %c>", response[itrial][0],
			response[itrial][1], response[itrial][2] );
		one=two=thr=0; /* place markers for key positions */
		if (type!='F') {one=2; two=3; thr=4;}
		else if (isupper(letter[itrial][0])) {one=0; two=1; thr=2;}
		else {one=4; two=5; thr=6;}

		/* letters l1 - l3 correct, a1 and a3 always contain outside targets */
		/* for 2 or 3 target case */


		l1=toupper(letter[itrial][one]); l2=toupper(letter[itrial][two]);
		l3=toupper(letter[itrial][thr]); a1=response[itrial][0];
		a2=response[itrial][1]; a3=response[itrial][2];
		if (num_targ==2) {a3=a2; /* shift response */ a2='*';}
		score_l2=score_s2=score_l3=score_s3=9; /* default */
		if (((l1==a1) & (l3==a3)) | ((l1==a3) & (l3==a1))) score_l2=2;
		else if ((l1==a1) | (l1==a3) | (l3==a1) | (l3==a3)) score_l2=1;
		else score_l2=0;
		if ((l1==a1) & (l3==a3)) score_s2=2;
		else if ((l1==a1) | (l3==a3)) score_s2=1;
		else score_s2=0;
		if (((l1==a1) & (l3==a2)) |
				((l1==a1) & (l3==a3)) |
				((l1==a2) & (l3==a1)) |
				((l1==a2) & (l3==a3)) |
				((l1==a3) & (l3==a1)) |
				((l1==a3) & (l3==a2))) score_l3=2;
		else if ((a1==l1)|(a1==l3)|(a2==l1)|(a2==l3)|(a3==l1)|(a3==l3))
			score_l3=1;
		else score_l3=0;
		if ((l1==a1) & (l2==a2) & (l3==a3)) score_s3=3;
		else if (score_s2==2) score_s3=2;
		else if ((score_s2==1) & (l2==a2)) score_s3=2;
		else if ((score_s2==1) | (l2==a2)) score_s3=1;
		else score_s3=0;

		fprintf (outfile, "[%d] l2:%d l3:%d s2:%d s3:%d", ident[trial], score_l2, score_l3, score_s2, score_s3);
		fprintf (outfile, " %d\n", lett_time); /* prior to rescoring */

		/* unpackage score cards for this trial type */
		if (ldl==1) {control=ctrl_ldl; control_score=s_scr_ldl;}
		else {control=ctrl_lll; control_score=s_scr_lll;}

		if (type=='C') {
			control++; /* one more control trial */
			if (score_l3==2) control_score++; /* no partial scoring */
		}
		if (control==Adjust_Rate) { /* time to step the rate */
			if (control_score==4) lett_time-=(Refresh*2);
			else if (control_score==3) lett_time-=Refresh;
			else if (control_score==1) lett_time+=Refresh;
			else if (control_score==0) lett_time+=(Refresh*2);

			control_score=control=0;
		} /* end scoring module */

		/* context save */
		if (ldl==1) {lett2_time=lett_time; ctrl_ldl=control; s_scr_ldl=control_score;}
		else {lett3_time=lett_time; ctrl_lll=control; s_scr_lll=control_score;}

		setcolor (3);
		if (itrial==(num_trials+1))
			strcpy (string, "Block completed.  Hit RETURN to continue:");
		else strcpy (string, "Hit RETURN to begin next trial:");
		outtextxy(10, 10, string);
		while (getch()!=13);
		cleardevice();
	}
	/* store data to file */
	closegraph();
	if (block==-2) inst(3);
	else if (block==-1) inst(5);
	/* end block */
}

void show_practice(
	FILE *infile,
	int block
)
{

	char letter[2][Num_Letters]; /* character array */
	char response[2][Num_Targ];  /* subject responses */
	int trial, position, i;
	char line[MAX_LINE];
	char string[100]; /* presentation window */
	int lett_time; /* current trial variables */

	for (trial=0; trial<2; trial++) {
		fgets(line,MAX_LINE,infile);
		position = 0;
		letter[trial][position] = (strtok(line, " \t\n"))[0];
		for (position=1; position<Num_Letters; position++)
			letter[trial][position] = (strtok(NULL, " \t\n"))[0];
	}

	setcolor(MSG_COLOR);
	/* if (block==-2) strcpy(string, "* Letter/Digit Condition *"); */
	/* else strcpy(string, "* Letter Only Condition *");            */
	/* outtextxy (10, 10, string); */
	strcpy(string, "You will now be given two example trials.  Hit RETURN:\n");
	outtextxy(10,40,string);
	while (getch()!=13);

	cleardevice();

	/*****/
	strcpy (string, "B C D F G H J K L M N P Q R S T V W X Y Z");
	outtextxy(10,10,string);
	strcpy (string, "b c d f g h j k l m n p q r s t v w x y z");
	outtextxy(10,40,string);
	strcpy (string, "0 1 2 3 4 5 6 7 8 9");
	outtextxy(10,70,string);
	while (getch()!=13);
	cleardevice();
	/*****/

	for (i=0; i<2; i++) {
		presentation(letter[i], LETT2_TIME);
		collect_data(response[i], block);

		strcpy (string, "Hit RETURN to continue:");
		outtextxy(10, 10, string);
		while (getch()!=13);

		cleardevice();
	}


	cleardevice();
}

/* vga graphics initialization routine */
void start_graphics() {
	int gdriver=VGA, gmode=VGAHI, errorcode;
	initgraph(&gdriver, &gmode, "C:\\BORLANDC\\BGI\\"); /* enter graphics mode */
	errorcode=graphresult();
	if (errorcode!=grOk) {
		printf("Graphics error:  %s\n", grapherrormsg(errorcode));
		printf("Press any key to halt:");
		getch();
		exit(1);
	} /* from BC++ Lib Ref, pg. 299 */
	settextstyle(SMALL_FONT,HORIZ_DIR,7);
	/* setusercharsize(); for future use; font size must be set to zero */
}

/* an error handler */

/* present a string of letters in serial fashion */
void presentation(
	char letter[],
	int lett_time
	)
	{
		int position;
		char string[2];

		/* fixation point */
		string[0]=' '; string[1]='\0'; /* set up character frame */
		ms_counter=0; while (ms_counter<1000);
		setcolor(Fix_Color); string[0]='+'; outtextxy(X, Y, string);
		ms_counter=0; while (ms_counter<1000);
		setcolor(Blank_Color); outtextxy (X, Y, string);
		ms_counter=0; while (ms_counter<500);

		for (position=0; position<Num_Letters; position++) {
			if (islower(letter[position])) {
				string[0]=toupper(letter[position]); setcolor(Field_Color); }
			else { string[0]=letter[position]; setcolor(Target_Color); }
			if (isdigit(letter[position])) setcolor(Target_Color);
			ms_counter=0; /* RESET TIMER */
			outtextxy(X, Y, string);             /* display character */
			/* wait for next mark */
			while (ms_counter<lett_time); ms_counter=0;  /* TIME PRESENTATION */
			setcolor (Blank_Color); /* background - mask */
			outtextxy(X, Y, string);
			while (ms_counter<MASK_TIME); /* TIME MASK */
		}
}
void collect_data( /* DATA COLLECTION ROUTINE */
	char response[],
	int block
)
{
	char string[100];
	int i;
	int num_targ=3;
	if ((block==-2) | (block==0) | (block==3)) num_targ=2;
	if (num_targ==2)
		strcpy (string, "Please enter the THREE target characters, in order: ");
	else strcpy (string, "Please enter the THREE target characters, in order: ");

	setcolor (MSG_COLOR);
	ms_counter=0;  while (ms_counter<1000);
	outtextxy(10, 10, string);


	response[2]='*'; /* default */
	for (i=0; i<3/* num_targ*/; i++) {
		response[i]=toupper(getch());
		/* escape hatch - hit '@' as a response */
		if (response[i]=='@') {stop_clock(); fcloseall(); closegraph(); exit(1);}
		if ((isalpha(response[i])==0)&(isdigit(response[i])==0)) response[i]='*';
	}
	if ((response[0]=='0')&(response[1]=='0')) {stop_clock(); fcloseall();
																							closegraph(); exit(1);}
	cleardevice();
}

void inst (
	int mssg
) {
	/* do good things, then */
	clrscr();
	if (mssg==1) {
	/* Text for 1st screen: */
	printf ("Thank you for participating in this experiment. Please read the\n");
	printf ("following instructions carefully. This experiment consists of four\n");
	printf ("blocks of experimental trials and two practice blocks. Each block consists\n");
	printf ("of trials containing a sequence of letter and digit target characters,\n");
	printf ("presented in the center of the screen. In each trial there are three\n");
	printf ("characters colored in RED, while the rest are colored in GREEN. Your\n");
	printf ("task is to report the identity of the three RED targets in each trial\n");
	printf ("in the ORDER in which they were presented. On some trials, the\n");
	printf ("three RED letter sequence will contain a digit (number). You must report\n");
	printf ("all RED characters including letters and digits. After each trial a prompt\n");
	printf ("will appear on the screen, and you will be asked to TYPE in the identity of\n");
	printf ("the three RED characters on the keyboard. Remember, you must type the\n");
	printf ("characters in the ORDER you think they appeared on the screen. By pressing the\n");
	printf ("RETURN now you can see two demonstration trials of mixed letter and digit\n");
	printf ("trials.\n");
	}

	else if (mssg==2) {
	/* Text for screen 2 (after demo block 1): */
	printf ("In all trials there are three RED targets. You must always make\n");
	printf ("three responses. If you are unsure about the identity of any character\n");
	printf ("then GUESS. You will now be given twenty practice trials in which all\n");
	printf ("target sequences contain both letters and digits. After each trial, report\n");
	printf ("the identity of all RED characters in the order they were presented.\n");

	}

	else if (mssg==3) {
	/* Text for screen 3 (after practice block 1) */
	printf ("This block of practice trials is now finished. You will now see\n");
	printf ("two demonstration trials in which there are three RED letters\n");
	printf ("as targets. Press the RETURN when you are ready.\n");
	}

	else if (mssg==4) {
	/* Text for screen 4 (after 3 letter demonstration trials) */
	printf ("You will now be given twenty practice trials where you will be\n");
	printf ("looking for THREE RED LETTER targets.  Report the identity of\n");
	printf ("the three red letters after each trial.  If you are unsure about\n");
	printf ("the identity of any of the letters then GUESS.  ALWAYS make THREE\n");
	printf ("RESPONSES and report ALL THREE RED LETTERS in the ORDER they were\n");
	printf ("presented.\n");
	}

	else if (mssg==5) {
	/* Text for screen 5 (after practice block 2) */
	printf ("The practice trials are finished. Please read carefully the following\n");
	printf ("additional instructions before completing the rest of the trials.\n");
	printf ("You will now be shown four more blocks of trials. On some trials, the\n");
	printf ("RED targets may contain the same character twice.\n");
	printf ("\nYOU MUST REPORT ALL RED CHARACTERS INCLUDING THE REPEATED ONES.\n");

	printf ("\nYour responses are NOT timed. If you get tired, please feel free\n");
	printf ("to rest after each block. If you have any questions, please ask the\n");
	printf ("experimenter now.\n");

	printf ("\nGOOD LUCK !\n");
	}

	else if (mssg==6) {
	printf (" ** BLOCK 1 ** \n");
	}

	else if (mssg==7) {
	printf (" ** BLOCK 2 ** \n");
	}

	else if (mssg==8) {
	printf (" ** BLOCK 3 ** \n");
	}

	else if (mssg==9) {
	printf (" ** BLOCK 4 ** \n");
	}

	printf ("\nHit RETURN to continue . . .\n");
	while (getch()!=13);
	clrscr();
}