#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <curses.h>

#define MAX_LINE 255
#define BUF_MAX 512

typedef struct sem_NODE {
  int id;
  int genre;
  int samples;
  int freq;
  char phon[30]; /**** reduce by allocating string space as needed ****/
  char stress[30]; /**** same ****/
} sem_node, *p_sem_node;

typedef struct lex_NODE {
  char grp;
  struct lex_NODE* next[27];
  p_sem_node latch;
} lex_node, *p_lex_node;

/* prototypes */
char *convert();
char *remap();
p_sem_node find();

p_lex_node lex_root;
  
main ()
{
  FILE *fp;
  char line[MAX_LINE];
  char *word, *freq;
  int wlen, i, j, k, count = 0, tic;
  p_lex_node lex_indx;
  p_sem_node wp; 

  printf("\nCompiling lexicon now.\n");
  fp = fopen ("clean_freq", "r");
  lex_root = (p_lex_node)malloc (sizeof (lex_node));
  for (i=0; i<27; i++) lex_root->next[i]=NULL;
  while ( fgets (line, MAX_LINE, fp) != NULL ) {
    lex_indx = lex_root;
    freq = strtok (line, " \t\n");
    word = strtok (NULL, " \t\n");
    remap(word);

    /* insert word into WAL; trace */
    lex_indx = lex_root;
    for (i=0; i<wlen; i++) {
      j=(int)word[i];
      if (lex_indx->next[j] == NULL) { 
	count++;
	lex_indx->next[j] = (p_lex_node) malloc (sizeof (lex_node));
        for (k=0; k<27; k++) lex_indx->next[k]=NULL;
      }
      lex_indx = lex_indx->next[j];
    }
    /* attach semantics */
    lex_indx->latch = (p_sem_node) malloc (sizeof (sem_node)); 
    tic++;
    if ((tic%10000)==0) {printf("."); fflush(stdout);}
    /* printf ("Terminal node reached.  Count is %d\n", count ); */
    /****  insert frequency information here ****/
  }
  fclose(fp);
  tic=0;
  printf("\nLexicon established.\nFrequency information integrated.\n");

  /* now that initial lexicon is loaded, we can begin attachments */
  /* and computations */

  fp=fopen("nettalk_nohead","r");

  while (fgets(line,MAX_LINE,fp)!=NULL) {
    word = remap(strtok(line," \t\n"));
    if ((wp=find(word))!=NULL) {
      strcpy(wp->phon,convert(strtok(NULL," \t\n")));
      strcpy(wp->stress,strtok(NULL," \t\n"));
    }
    tic++;
    if ((tic%10000)==0) {printf("."); fflush(stdout);}
  }
  fclose(fp);
  printf("\nPhonetic and stress information integrated.\n");
  printf("Lexicon compiled.\n");
}

p_sem_node find(word)
  char word[];
{
  int i;
  p_lex_node trace;

  trace = lex_root; /* initialize */
  for (i=0; i<strlen(word); i++) {
    if (trace->next[word[i]]!=NULL)
     trace=trace->next[word[i]];
    else return(NULL); /* word search failed */
  }
  return(trace->latch);
}

char *remap(word)
  char word[];
{
  int i;

  /* graphemes remapped to indices; computational aid */
  for (i=0; i<strlen(word); i++) {
    if (( word[i] >= 'a' ) & ( word[i] <= 'z' ))
      word[i] = word[i] - 'a';
    else if (( word[i] >= 'A' ) & ( word[i] <= 'Z' ))
      word[i] = word[i] - 'A';
    else if (word[i]=='-') word[i] = 26; /* only special char */
    else {
      printf ("Error; non-alphabetic character in lexicon\n");
      exit(1);
    }
  }
} /* end remap */

char *convert(s)
  char s[];
{
  int i, index;
  char x1[2], x2[2];
  static char r[50];

  strcpy(x1,"+");
  strcpy(x2,"+");
  strcpy(r,"");
  for (i=0; i<strlen(s); i++) {
    switch(s[i]) {
      case 'a': {strcat(r,"a"); break;}
      case 'b': {strcat(r,"b"); break;}
      case 'c': {x1[0]=141; strcat(r,x1);}
      case 'd': {strcat(r,"d"); break;}
      case 'e': {strcat(r,"e"); break;}
      case 'f': {strcat(r,"f"); break;}
      case 'g': {strcat(r,"g"); break;}
      case 'h': {strcat(r,"h"); break;}
      case 'i': {strcat(r,"i"); break;}
      case 'k': {strcat(r,"k"); break;}
      case 'l': {strcat(r,"l"); break;}
      case 'm': {strcat(r,"m"); break;}
      case 'n': {strcat(r,"n"); break;}
      case 'o': {strcat(r,"o"); break;}
      case 'p': {strcat(r,"p"); break;}
      case 'r': {strcat(r,"r"); break;}
      case 's': {strcat(r,"s"); break;}
      case 't': {strcat(r,"t"); break;}
      case 'u': {strcat(r,"u"); break;}
      case 'v': {strcat(r,"v"); break;}
      case 'w': {strcat(r,"w"); break;}
      case 'x': break;
      case 'y': {strcat(r,"j"); break;}
      case 'z': {strcat(r,"z"); break;}
      case 'A': {x1[0]= 73; strcat(r,x1); break;}
      case 'C': {x1[0]= 83; strcat(r,x1); break;}
      case 'D': {x1[0]= 68; strcat(r,x1); break;}
      case 'E': {x1[0]= 69; strcat(r,x1); break;}
      case 'G': {x1[0]= 78; strcat(r,x1); break;}
      case 'I': {x1[0]= 73; strcat(r,x1); break;}
      case 'J': {x1[0]= 90; strcat(r,x1); break;}
      case 'K': break;
      case 'L': {x1[0]= 56; strcat(r,"l"); strcat(r,x1); break;}
      case 'M': {x1[0]= 56; strcat(r,"m"); strcat(r,x1); break;}
      case 'N': {x1[0]= 56; strcat(r,"n"); strcat(r,x1); break;}
      case 'O': {x1[0]=141; x2[0]= 73; strcat(r,x1); strcat(r,x2); break;}
      case 'Q': break;
      case 'R': break;
      case 'S': {x1[0]= 83; strcat(r,x1); break;}
      case 'T': {x1[0]= 84; strcat(r,x1); break;}
      case 'U': break;
      case 'W': {x1[0]= 85; strcat(r,"a"); strcat(r,x1); break;}
      case 'X': {strcat(r,"ks"); break;}
      case 'Y': {x1[0]= 83; strcat(r,x1); break;}
      case 'Z': {x1[0]= 90; strcat(r,x1); break;}
      case '@': {x1[0]= 81; strcat(r,x1); break;}
      case '!': {strcat(r,"ts"); break;}
      case '#': {strcat(r,"gz"); break;}
      case '*': {strcat(r,"hw"); break;}
      case '^': {x1[0]=195; strcat(r,x1); break;}
      case '+': {strcat(r,"ua"); break;}
      default : break;
      }
  }
  return(r);
}
