/* prototype word pair scoring system */
/* 6/16/93 scott cost */

/* 6/17/93 this will be an addition-then-deletion tree */
/* only two string primitive: addition and deletion */
/* must be executed in the sequence a*d*.  This will */
/* prevent endless descent, and is sufficient for all */
/* rearrangments */

#include <stdio.h>
#define WSIZE 20
#define MAX_DEPTH 20

typedef struct SOLN { /* solution to transformation */
  char action;        /* best to date, null action signifies end */
  char grapheme;      
  int posn;

  char c_action;      /* current working solution */
  char c_grapheme;
  int c_posn;
} Soln, *Psoln;

char *del(); /* primitive string operations */
char *add();
int contain(); 

main(
int argc,
char* argv[]
) 
{
  Soln soln[MAX_DEPTH];
  char stim[WSIZE], resp[WSIZE], template[WSIZE];

  strcpy (stim, argv[1]); /* copy word pair into working buffers */
  strcpy (resp, argv[2]);
  strcpy (template, argv[2]);

  score (stim, resp, template, 0, 0); /* mode: 0=addition, 1=deletion */
  /* printf ("Done.\n"); */
}

int score (
  char stim[],     /* stimulus */
  char resp[],     /* response */
  char template[], /* marker for response words */
  int level,       /* # primitive operations */
  int mode         /* addn/deln mode */
)
{
  int length1, length2, depth, i, j;
  char temp[WSIZE]; /* temp working buffer */
  char hold;

  /* printf ("Score, level %d:  %s -> %s\n", level, stim, resp); */

  length1 = strlen (stim);
  length2 = strlen (resp);

  if (strcmp (stim, resp) == 0) { /* solution recovered */
    /* printf ("Hit!!! at level %d\n", level); */
    
    return;
  }

  /* ARBITRARY MAXIMUM DEPTH; TEMPORARY ONLY!!! */
  if (level == 5) return; /* max depth necessary for conversion */ 

  if ((mode == 0) &&        /* add if deletion sequence not begun and */
      (level < length2)) { /* word not yet completely replaced */
    /* primitive #2: addition */
    for (i=0; i<=length1; i++)        /* cycle through positions */
      for (j=0; j<length2; j++) {

	/* Template functions to enforce the following restriction: */
	/* additions may be used up to the point of adding the entire */
	/* response (to cover case of complete change), but may use */
	/* each letter in the response only once. */

	if (template[j] == 0) continue; /* skip letters used above */
	/* set up for reduced string substitution */
	hold = template[j]; template[j] = 0; /* mark letter in use */
	/* do not add anything new to response */
	strcpy (temp, stim);
	
	/* Mark solution trail */
	sol
	
	score (add (temp, i, resp[j]-'a', length1), resp, template,level+1, 0);
	template[j] = hold; /* restore for next pass */
      }
  }

  /* primitive #1: deletion */
  if (length2 < length1) /* prevent meaningless reduction */ 
    for (i=0; i<length1; i++) {
      strcpy (temp, stim); /* allow nondestructive modification of resp */
      score (del (temp, i, length1), resp, template,level+1, 1); 
    }
}

char *del (
char* stim,
int posn,
int len
)
{
  int i;

  for (i=posn; i<len; i++) stim[i] = stim[i+1];
  return (stim);
}

char *add (
char* stim,
int posn,
int chr,
int len
)
{
  int i;

  for (i=len; i>=posn; i--)
    stim[i+1] = stim[i];
  stim[posn] = chr + 'a';
  return (stim);
}

/* currently unused */
/* is <chr> contained in set <string> */
int contain (
char string[],
int chr,
int len
)
{
  int i;
 
  chr += 'a';
  for (i=0; i<len; i++)
    if (string[i] == chr) return (1);
  return(0);
}







